package core

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file core_date_intf.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
open! Import

module Date = Core_kernel.Date

module type Core_date = sig
  type t = Date.t
  include module type of Date with type t := t

  (** This formats a date using the format patterns available in [strftime]. *)
  val format : t -> string -> string

  (** This parses a date using the format patterns available in [strptime]. *)
  val parse : fmt:string -> string -> t

  val of_tm : Core_unix.tm -> t
end